#ifndef __DATA_H__
#define __DATA_H__
#include "main.h"

// typedef unsigned long int simplex;
typedef int vertex;
typedef int simplex;

class flip;
class flips;
class vertices_lookup;


// vertices of a simplex
class vertices: public std::set<vertex,std::less<vertex> >
{
 private:
  static int n;  
  static int d;  
  static vertices_lookup lookup;  
 protected:
 public:  
  const simplex orden() const;
  const vertices & reverse_orden(const simplex &); // uses lookup-table
  void set_dimensions(int N, int D);  
  friend std::ostream & operator << (std::ostream &, const vertices &);
  friend bool operator==(const vertices &, const vertices &); 
};

// tables used by vertices to make computation faster
class vertices_lookup 
{
 private:
  int n,d;  
  std::vector<vertices> ReverseOrden;
  std::vector<std::vector<int> > fast_binomial;  
 protected:
  vertices manual_reverse_orden(const simplex &) const;   
 public:
  void generate_tables(int N, int D);  
  const vertices & reverse_orden(const simplex &) const;     
  int get_binomial(int i, int j) const;  
};


// total order on the vertices
struct vertices_order
{
  bool operator() (const vertices &, const vertices &) const;
};


// simplical complex, data compressed by orden
class compact_simplices: public std::vector<simplex>
{
 private:
 protected:  
 public:
  compact_simplices();
  virtual ~compact_simplices();  
  hash_value hash_function() const;
  friend std::istream & operator >> 
    (std::istream &, compact_simplices &);
  friend std::ostream & operator << 
    (std::ostream &, const compact_simplices &);
  friend const bool operator==
    (const compact_simplices &, const compact_simplices &);  
  bool starshaped() const; 
};

// same but with complete vertex information
class simplices: public compact_simplices
{
 private:
  std::vector<vertices> v;  
 protected:
  void orden();  
  void reverse_orden();  
 public:
  simplices() { };  
  simplices(const std::set<vertices,vertices_order> &);  
  simplices(const compact_simplices &);
  virtual ~simplices();
  const std::vector<vertices> & get_vertices() const { return(v); }  
  friend std::ostream & operator << (std::ostream &, const simplices &);
};

class flip
{
 private: 
  std::vector<vertices> deltaplus,deltaminus;
 protected:
 public:
  flip();  
  flip(std::istream &);
  flip(const flip &);  
  virtual ~flip();
  flip & operator =(const flip &);  
  const std::vector<vertices> & get_deltaplus()  const { return deltaplus; };
  const std::vector<vertices> & get_deltaminus() const { return deltaminus;};
  friend std::ostream & operator <<(std::ostream &, const flip &);
  void mirror();  
};

class flips: public std::vector<flip>
{
 protected:
  void sort_flips_containing();  
 public:
  flips();
  virtual ~flips();
  friend std::istream & operator >> (std::istream &, flips &);
};


class goodcircuit
{
 private:
  std::vector<std::vector<vertices> > supported;
  flip supporter;  
  std::vector<std::set<vertices,vertices_order> > link;  
  std::set<vertices,vertices_order> bistellarneighbor;  
  bool good;  
 public: 
  goodcircuit(const simplices &, const flip &);
  bool is_good() const { return(good); }  
  void do_flip(const simplices &, const flip &);  
  simplices get_neighbor() const { return(bistellarneighbor); }  
};  




#endif
